import logging
import os

BASE_DIR = os.path.dirname(os.path.abspath(__file__))


PROFILE_DIRS = {
    'windows_old': '~\\Application Data\\Thunderbird\\Profiles\\',
    'windows': '~\\AppData\\Roaming\\Thunderbird\\Profiles\\',
    'linux': '~/.thunderbird/',
    'linux_other': '~/.mozilla-thunderbird/',
    'osx': '~/Library/Thunderbird/Profiles/',
}


OUTBOX_FOLDERS = ['Sent', 'Wysłane']


logging_config = dict(
    version=1,
    disable_existing_loggers=False,
    formatters={
        'default': {
            'format': '[{levelname}] [{asctime}]: {message}',
            'style': '{'
        },
        'plain': {
            'format': '{message}',
            'style': '{'
        }
    },
    handlers={
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'default',
            'level': logging.DEBUG
        },
        'file': {
            'class': 'logging.FileHandler',
            'formatter': 'plain',
            'level': logging.INFO,
            'filename': os.path.join(BASE_DIR, 'info.log'),
            'mode': 'w',
        },
    },
    loggers={
        'emails_gdpr': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True
        },
        'emails_gdpr.file': {
            'handlers': ['file'],
            'level': 'INFO',
            'propagate': True
        }
    },
)



import inspect
import logging


flogger = logging.getLogger('emails_gdpr.file')
logger = logging.getLogger('emails_gdpr')


def starting():
    logger.info('Starting %s...' % inspect.stack()[1][3])


def finished():
    logger.info('...Finished %s' % inspect.stack()[1][3])


def outbox_found(f_path):
    logger.info('Found outbox at: %s' % f_path)


def email_found(email, outbox):
    logger.info('Found email address (%s) for %s' % (email, outbox))


def processing_msg(author, recipient):
    logger.info('Processing message (from: %s, to: %s).' % (author, recipient))


def email_dont_match():
    logger.info('Neither author nor recipients emails match any of your emails.')


def mailbox_results(mbox):
    flogger.info(str(mbox))



import configparser
import mailbox
import os
import platform

from emails_gdpr import exc, imap_utf7, settings, logs, utils


class MailBox:
    def __init__(self, email):
        self.email = email
        self.emails_in = set()
        self.emails_out = set()
        self.domains_in = set()
        self.domains_out = set()

        self.emails_gdpr = set()
        self.domains_gdpr = set()

    def __repr__(self):
        return 'MailBox(%s)' % self.email

    def __str__(self):
        return self.build_str_emails() + self.build_str_domains()

    def add_outgoing(self, recipient_email):
        self.emails_out.add(recipient_email)
        self.domains_out.add(utils.extract_tld(recipient_email))

    def add_incoming(self, author_email):
        self.emails_in.add(author_email)
        self.domains_in.add(utils.extract_tld(author_email))

    def process(self):
        self.emails_gdpr = self.emails_in - self.emails_out
        self.domains_gdpr = self.domains_in - self.domains_out

    def build_str_emails(self):
        s = 'Emails to GDPR for %s:\n' % self.email
        if not self.emails_gdpr:
            return '%s No emails yet.\n' % s

        for email in self.emails_gdpr:
            s += '\t- %s\n' % email
        return s

    def build_str_domains(self):
        s = 'Domains to GDPR for %s:\n' % self.email
        if not self.domains_gdpr:
            return '%s No domains yet.\n' % s

        for domain in self.domains_gdpr:
            s += '\t- %s\n' % domain
        return s


def discover_config_dir():
    logs.starting()

    os_platform = platform.system()
    for os_name, config_dir in settings.PROFILE_DIRS.items():
        if not os_platform.lower() in os_name:
            continue

        path = os.path.expanduser(config_dir)
        ini_path = os.path.join(path, 'profiles.ini')
        if not os.path.exists(ini_path):
            continue
        logs.finished()
        return path

    raise exc.ConfigNotFound('No config directory found.')


def discover_profile_dir(config_dir):
    logs.starting()

    parser = configparser.ConfigParser()
    parser.read(os.path.join(config_dir, 'profiles.ini'))
    for section in parser.sections():
        if 'Profile' not in section:
            continue

        logs.finished()
        return os.path.join(config_dir, parser[section]['path'])

    raise exc.ProfileNotFound('No profiles found.')


def discover_my_emails(profile_dir):
    logs.starting()

    outboxes = []
    for root, dirs, files in os.walk(profile_dir):
        for f_path in files:
            if f_path.endswith(('.dat', '.msf')):
                continue

            f_path_decoded = imap_utf7.decode(f_path.encode())
            is_outbox = any(f_name in f_path_decoded
                            for f_name in settings.OUTBOX_FOLDERS)
            if not is_outbox:
                continue

            logs.outbox_found(f_path)
            outboxes.append(os.path.join(root, f_path))

    if not outboxes:
        raise exc.OutboxNotFound('No outbox folders found')

    emails = []
    for outbox in outboxes:
        try:
            _, msg = mailbox.mbox(outbox).popitem()
        except KeyError:
            # If outbox is empty
            continue

        email_n = utils.email_norm(msg['from'])
        if not email_n:
            continue

        logs.email_found(email_n, outbox)
        emails.append(email_n)

    logs.finished()
    return emails


def get_all_messages(db_path):
    logs.starting()

    sql = "SELECT DISTINCT c3author, c4recipients FROM messagesText_content"
    with utils.get_db_conn(db_path) as cur:
        cur.execute(sql)
        messages = cur.fetchall()

    logs.finished()
    return messages


def parse_recipients(recipients):
    if ',' in recipients:
        recipients = recipients.split(',')

    if isinstance(recipients, str):
        recipients = {recipients}

    result = set()
    for recipient in recipients:
        email_n = utils.email_norm(recipient.strip())
        if not email_n:
            continue
        result.add(email_n)
    return result


def generate_mailboxes(messages, my_emails):
    mailboxes = {my_email: MailBox(my_email) for my_email in my_emails}

    for message in messages:
        author = utils.email_norm(message['c3author'])
        if not author:
            continue

        for recipient in parse_recipients(message['c4recipients']):
            logs.processing_msg(author, recipient)
            if author in my_emails:
                mailboxes[author].add_outgoing(recipient)
                continue

            if recipient in my_emails:
                mailboxes[recipient].add_incoming(author)
                break
            logs.email_dont_match()

    logs.finished()
    return mailboxes


def main():
    config_dir = discover_config_dir()
    profile_dir = discover_profile_dir(config_dir)
    my_emails = discover_my_emails(profile_dir)

    with utils.make_db_copy(profile_dir) as db_path:
        messages = get_all_messages(db_path)

    mailboxes = generate_mailboxes(messages, my_emails)

    for mbox in mailboxes.values():
        mbox.process()
        logs.mailbox_results(mbox)


if __name__ == '__main__':
    main()



import os
import shutil
import sqlite3
import unicodedata
from contextlib import contextmanager

import tldextract
from cachetools import cached, LRUCache
from email_normalize import normalize as norm_email

cache = LRUCache(maxsize=500)
        

@cached(cache)
def email_norm(email):
    try:
        return norm_email(email, resolve=False)
    except ValueError:
        print('Error parsing email %s' % email)


@cached(cache)
def extract_tld(email):
    return tldextract.extract(email).registered_domain


def norm_path(path):
    return unicodedata.normalize('NFD', path)


@contextmanager
def get_db_conn(db_path):
    conn = sqlite3.connect(db_path)
    conn.row_factory = sqlite3.Row

    try:
        cur = conn.cursor()
        yield cur
    finally:
        cur.close()

    conn.close()


@contextmanager
def make_db_copy(profile_dir):
    db_path = os.path.join(profile_dir, 'global-messages-db.sqlite')
    db_copy_path = os.path.join(profile_dir, 'global-messages-db-copy.sqlite')
    shutil.copyfile(db_path, db_copy_path)

    try:
        yield db_copy_path
    finally:
        os.remove(db_copy_path)



class GDPREmails(Exception):
    pass


class ConfigNotFound(GDPREmails):
    """ Exception raises when no profiles.ini directory is found."""
    pass


class ProfileNotFound(GDPREmails):
    """ Exception raises when no profiles are defined in profiles.ini file."""
    pass


class OutboxNotFound(GDPREmails):
    """
    Exception raises when no outbox folders containing pre-defined names
    are found in a profile directory.
    """
    pass



# Source code from: https://github.com/MarechJ/py3_imap_utf7

# encoding
import binascii


def modified_base64 (s):
    s = s.encode('utf-16be')
    return binascii.b2a_base64(s).rstrip(b'\n=').replace(b'/', b',')


def doB64(_in, r):
    if _in:
        r.append(b'&' + modified_base64(''.join(_in)) + b'-')
    del _in[:]


def encode(s:str):
    r = []
    _in = []
    for c in s:
        ordC = ord(c)
        if 0x20 <= ordC <= 0x25 or 0x27 <= ordC <= 0x7e:
            doB64(_in, r)
            r.append(c.encode())
        elif c == '&':
            doB64(_in, r)
            r.append(b'&-')
        else:
            _in.append(c)
    doB64(_in, r)
    return b''.join(r)


# decoding
def modified_unbase64(s):
    #b = b64decode(s, b"+,")
    b = binascii.a2b_base64(s.replace(b',', b'/') + b'===')
    return b.decode('utf-16be')


def decode(s:bytes):
    r = []
    decode = bytearray()
    for c in s:
        if c == ord('&') and not decode:
            decode.append(ord('&'))
        elif c == ord('-') and decode:
            if len(decode) == 1:
                r.append('&')
            else:
                ab = modified_unbase64(decode[1:])
                r.append(ab)
            decode = bytearray()
        elif decode:
            decode.append(c)
        else:
            r.append(chr(c))

    if decode:
        r.append(modified_unbase64(decode[1:]))

    bin_str = ''.join(r)
    return bin_str



import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('requirements.txt') as requirements_file:
    required = requirements_file.read().splitlines()

setuptools.setup(
    name="emails-gdpr",
    version="1.0.5",
    author="Damian P.",
    author_email="an0o0nyme@gmail.com",
    description="",
    keywords="emails gdpr data protection",
    install_requires=required,
    license="MIT license",
    long_description=long_description + '\n\n' + history,
    long_description_content_type="text/markdown",
    url="https://github.com/an0o0nym/emails-gdpr",
    packages=setuptools.find_packages(),
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Natural Language :: English',
    ],
    entry_points = {
        'console_scripts': [
            'emails-gdpr=emails_gdpr.main:main',
        ],
    }
)